import io
from pathlib import PosixPath

import requests
import validators
from PIL import Image


class ImageRenderer:
    # TODO: Если это не изображение
    # TODO: Проверка формата
    # TODO: Проверка размера

    def __init__(self, content):
        self.read_content(content)

    @classmethod
    def new(cls, size, color=(255, 255, 255, 0)):
        image = Image.new("RGBA", size, color=color)
        return cls(image)

    @property
    def info(self):
        return {
            "format": self.image.format,
            "size": self.image.size,
            "mode": self.image.mode,
            "mime": self.image.get_format_mimetype(),
        }

    def read_content(self, content):
        if isinstance(content, str) and validators.url(content):
            self.image = Image.open(io.BytesIO(requests.get(content).content))
        elif isinstance(content, str) or isinstance(content, PosixPath):
            self.image = Image.open(content)
        elif isinstance(content, io.BufferedReader):
            self.image = Image.open(content)
        elif isinstance(content, bytes):
            self.image = Image.open(io.BytesIO(content))
        elif isinstance(content, Image.Image):
            self.image = content

        if not self.image:
            return

        if self.image.format != "PNG" or self.image.mode != "RGBA":
            self.convert()

    def set_transparency(self, transparency=127):
        assert self.image.mode == "RGBA", "Изображение не в RGBA"
        self.image.putalpha(transparency)

    def paste(self, im2, position=(0, 0)):
        assert isinstance(
            im2, self.__class__
        ), f"Объект долженн быть класса {self.__class__.__name__}"
        self.image.paste(im2.image, position, im2.image)

    @property
    def content(self):
        with io.BytesIO() as f:
            self.image.save(f, format=self.image.format, quality=100, subsampling=0)
            return f.getvalue()

    def resize(self, size):
        self.image.thumbnail(size)

    def convert(self, format="PNG"):
        with io.BytesIO() as f:
            self.image = self.image.convert("RGBA")
            self.image.save(f, format=format, quality=100, subsampling=0)
            self.read_content(f.getvalue())



import os
from pathlib import Path
from typing import Optional

from pydantic import BaseSettings, DirectoryPath

BASEPATH = Path(os.path.dirname(os.path.realpath(__file__))).parent


class GeneralSettings(BaseSettings):
    TEMPLATES_PATH: DirectoryPath = (BASEPATH / "templates").resolve()
    MONTH_SALE_RATE: int = 9
    TURNOVER_RATE: int = 5


class NotionSettings(BaseSettings):
    NOTION_SECRET_TOKEN: str
    CACHE_ENABLED: bool = True


class FeedSettings(BaseSettings):
    PHONE: str = "+7 499 384 44 03"
    ADDRESS: str = "Москва, ул. Крупской, 4к1"
    MANAGER_NAME: str = "Менеджер магазина"
    SHOP_NAME: str = "Grey Dream Horse Shop (Конный магазин)"
    COMPANY_NAME: str = "GD Horse Shop (Конный магазин)"
    SHOP_URL: str = "https://www.instagram.com/gd_horse_shop/"
    ULA_CATEGORY_ID: int = 5
    ULA_SUBCATEGORY_ID: int = 507
    AVITO_CATEGORY: str = "Товары для животных"


class S3Settings(BaseSettings):
    ENDPOINT_URL: str = "https://storage.yandexcloud.net"
    BUCKET_NAME: str = "gdshop"
    ACCESS_KEY: str
    SECRET_KEY: str


class CacheSettings(BaseSettings):
    CACHE_CLASS: str = "KeyDBCache"
    CACHE_DSN: Optional[str]
    CACHE_HSTORE: str = "notion"
    CACHE_SYSTEM_HSTORE: str = "gdshoplib"


class ProductSettings(BaseSettings):
    PRODUCT_DB: str = "2d1707fb-877d-4d83-8ae6-3c3d00ff5091"


class PriceSettins(BaseSettings):
    # Базовые коэфиценты цены
    PRICE_VAT_RATIO: float = 0.16
    PRICE_NEITRAL_RATIO: float = 0.40
    PRICE_PROFIT_RATIO: float = 0.60
    EURO_PRICE: int = 72


class RecurrentCommonExpenseSettings(BaseSettings):
    # Общие месячные расходы
    CLOUD_HOSTING_MONTH: int = 2400
    PLATFORMS_MONTH: int = 2000
    AD_MONTH: int = 3000  # Рассход на общие рекламные компании

    def all(self):
        return sum([self.CLOUD_HOSTING_MONTH, self.PLATFORMS_MONTH, self.AD_MONTH])


class RecurrentSKUExpenseSettings(BaseSettings):
    # Расходы на каждую товарную позицию
    AVITO_SKU_PRICE: int = (
        2495 / 83
    )  # (общее количество затрат на просмотры в месяц) / (количество товаров)
    ULA_SKU_PRICE: int = 20
    PACKING_PRICE: int = 20

    def all(self):
        return sum([self.AVITO_SKU_PRICE, self.ULA_SKU_PRICE])


class OnceSKUExpenseSettings(BaseSettings):
    # Разовые расходы на каждый SKU
    DESCRIPTION_PRICE: int = 125

    def all(self):
        return sum([self.DESCRIPTION_PRICE])


class OnceProductExpenseSettings(BaseSettings):
    # Разовые расходы на каждый товар
    STOCK_DELIVERY_PRICE: int = 200 / 20

    def all(self):
        return sum([self.STOCK_DELIVERY_PRICE])


class VKSettings(BaseSettings):
    VK_BASEPATH: str = "https://api.vk.com/method/"
    VK_API_VERSION: str = "5.131"
    VK_GROUP_ID: str = "215870481"
    VK_CLIENT_ID: str = "51521300"
    VK_USER_SCOPE: str = "notify,friends,photos,audio,video,stories,pages,status,notes,wall,ads,offline,docs,groups,notifications,stats,email,market"
    VK_SECRET_KEY: Optional[str]
    VK_USER_ACCESS_TOKEN: Optional[str]


class AVITOSettings(BaseSettings):
    AVITO_CLIENT_ID: str = "PjsDDbv2OD294pmW1aak"
    AVITO_SECRET: str



import datetime

from lxml import etree, objectify

from gdshoplib.apps.product import Product
from gdshoplib.core.settings import FeedSettings
from gdshoplib.packages.s3 import S3, SimpleS3Data


class Feed:
    DESCRIPTION_TEMPLATE = "basic.txt"
    KEY = "basic"

    @classmethod
    def get_platform_class(cls, key):
        for platform in cls.__subclasses__():
            if platform.KEY.lower() == key.lower():
                return platform
        return cls

    def __init__(self):
        self.feed_settings = FeedSettings()
        self.root = self.get_root()

    def get_root(self):
        root = etree.Element("yml_catalog")
        objectify.deannotate(root, cleanup_namespaces=True, xsi_nil=True)
        root.attrib["date"] = str(datetime.datetime.now())
        return root

    def get_shop(self):
        shop = objectify.Element("shop")

        shop.name = self.feed_settings.SHOP_NAME
        shop.company = self.feed_settings.COMPANY_NAME
        shop.url = self.feed_settings.SHOP_URL
        currencies = objectify.Element("currencies")
        currency = etree.Element("currency")
        currency.attrib["id"] = "RUB"
        currency.attrib["rate"] = "1"
        objectify.deannotate(currency, cleanup_namespaces=True, xsi_nil=True)
        objectify.deannotate(currencies, cleanup_namespaces=True, xsi_nil=True)
        currencies.append(currency)
        shop.currencies = currencies
        objectify.deannotate(shop, cleanup_namespaces=True, xsi_nil=True)

        return shop

    def get_offers(self, products):
        offers = etree.Element("offers")
        objectify.deannotate(offers, cleanup_namespaces=True, xsi_nil=True)
        for product in products:
            offers.append(self.get_offer(product))

        return offers

    @classmethod
    def get_old_price(cls, product):
        if (
            product.price.now <= product.price.profit / 1.05
        ):  # Старую цену можно указывать, только если разница больше 5%
            return product.price.profit

    def get_offer(self, product):
        appt = objectify.Element("offer")
        appt.attrib["id"] = product.sku
        appt.attrib["available"] = str(product.available()).lower()
        appt.currencyId = "RUB"
        appt.price = product.price.now
        appt.title = product.name
        appt.name = product.name
        appt.vendor = product.brand.title
        appt.delivery = "true"
        appt.description = product.description.generate(self)

        if self.get_old_price(product):
            appt.oldprice = self.get_old_price(product)

        for image in product.images:
            appt.addattr("picture", image.get_url())

        objectify.deannotate(appt, cleanup_namespaces=True, xsi_nil=True)
        return appt

    def get_feed(self):
        shop = self.get_shop()
        shop.append(self.get_offers(self.products))
        self.root.append(shop)
        return etree.tostring(
            self.root,
            pretty_print=True,
            encoding="utf-8",
            xml_declaration=True,
            standalone=True,
        )

    @property
    def products(self):
        return Product.query(filter=self.product_filter)

    @property
    def product_filter(self):
        return dict(status_description="Готово", status_publication="Публикация")

    def get_s3_container(self, content):
        return S3(
            SimpleS3Data(content, f"feed.{self.KEY.lower()}", mime="application/xml")
        )

    def push_feed(self):
        s3 = self.get_s3_container(self.get_feed())
        s3.put()



import re

import requests

from gdshoplib.packages.renderer import ImageRenderer
from gdshoplib.packages.s3 import S3
from gdshoplib.services.notion.block import Block
from gdshoplib.services.notion.page import Page


class ProductMedia(Block):
    def __init__(self, *args, **kwargs):
        super(ProductMedia, self).__init__(*args, **kwargs)
        self.response = None
        self.s3 = S3(self)

    @property
    def url(self):
        return self[self.type]["file"]["url"]

    @property
    def badges(self):
        result = []
        for badge in self.parent.badges:
            if not badge.work or not badge.file:
                continue
            result.append(badge)
        return result

    @property
    def key(self):
        return self.notion.get_capture(self) or f"{self.type}_general"

    def fetch(self):
        self.access() or self.refresh()
        self.s3.get() or self.s3.put()

        return self.s3.get()

    def get_url(self):
        return f"{self.s3.s3_settings.ENDPOINT_URL}/{self.s3.s3_settings.BUCKET_NAME}/{self.file_key}"

    @property
    def file_key(self):
        return f"{self.parent.sku}.{self.id}.{self.format}"

    def access(self):
        return requests.get(self.url).ok

    def exists(self):
        return self.s3.exists()

    @property
    def name(self):
        pattern1 = re.compile(r".*\/(?P<name>.*)")
        r = re.findall(pattern1, self.url)
        if not r or not r[0]:
            return None
        return r[0].split("?")[0]

    @property
    def format(self):
        pattern = re.compile(r"\/.*\.(\w+)(\?|$)")
        r = re.findall(pattern, self.url)
        return r[0][0] if r else None

    def request(self):
        response = requests.get(self.url)
        if not response.ok:
            raise MediaContentException
        return response

    def get_content(func):
        def wrap(self, *args, **kwargs):
            if not self.response:
                if not self.access():
                    self.refresh()
                self.response = self.request()
            return func(self, *args, **kwargs)

        return wrap

    @property
    @get_content
    def content(self):
        return self.response.content

    @property
    @get_content
    def hash(self):
        return self.response.headers.get("x-amz-version-id")

    @property
    @get_content
    def mime(self):
        return self.response.headers.get("content-type")

    def get_badge_coordinates(self, badge):
        assert isinstance(badge, Page)
        result = [int(point.strip()) for point in badge.coordinates.split(",")]
        return result

    def get_size(self, badge):
        assert isinstance(badge, Page)
        result = [int(point.strip()) for point in badge.size.split(",")]
        return result

    def apply_badges(self):
        if self.type not in ("image",):
            return self

        product_image = ImageRenderer(self.content)
        size = tuple(size + 50 for size in product_image.image.size)
        result = ImageRenderer.new(size)
        result.paste(product_image, (50, 50))
        for badge in self.badges:
            if not requests.get(badge.file).ok:
                badge.refresh()

            _badge = ImageRenderer(badge.file)
            if badge.size:
                _badge.resize(self.get_size(badge))
            if badge.transparency:
                _badge.set_transparency(badge.transparency)
            result.paste(_badge, self.get_badge_coordinates(badge))
        return result


class MediaContentException(Exception):
    ...



from typing import Optional

import boto3
from botocore.exceptions import ClientError
from pydantic import BaseModel

from gdshoplib.core.settings import S3Settings


class S3:
    def __init__(self, data):
        self.data = data
        self.content = None
        self.s3_settings = S3Settings()
        self.__session = None
        self.__client = None

    @property
    def session(self):
        if not self.__session:
            self.__session = boto3.session.Session()

        return self.__session

    @property
    def s3(self):
        if not self.__client:
            self.__client = self.session.client(
                service_name="s3",
                endpoint_url=self.s3_settings.ENDPOINT_URL,
                aws_access_key_id=self.s3_settings.ACCESS_KEY,
                aws_secret_access_key=self.s3_settings.SECRET_KEY,
            )
        return self.__client

    def put(self):
        return self.s3.put_object(
            Bucket=self.s3_settings.BUCKET_NAME,
            Key=self.data.file_key,
            Body=self.data.content,
            ACL="public-read",
            StorageClass="COLD",
            ContentType=self.data.mime,
        )

    def exists(self):
        try:
            self.s3.head_object(
                Bucket=self.s3_settings.BUCKET_NAME, Key=self.data.file_key
            )
        except ClientError:
            return False
        else:
            return True

    def get(self):
        try:
            return self.s3.get_object(
                Bucket=self.s3_settings.BUCKET_NAME, Key=self.data.file_key
            )
        except ClientError as ex:
            if ex.response["Error"]["Code"] == "NoSuchKey":
                return None
            raise ex

    def delete(self):
        return self.s3.delete_object(
            Bucket=self.s3_settings.BUCKET_NAME, Key=self.data.file_key
        )

    def __str__(self) -> str:
        return f"{self.__class__}: {self.data.file_key}"

    def __repr__(self) -> str:
        return f"{self.__class__}: {self.data.file_key}"


class FileInfoModel(BaseModel):
    id: str
    format: str
    prefix: Optional[str]


class SimpleS3Data:
    def __init__(
        self, content, /, file_key, mime, file_info: FileInfoModel = None, parent=None
    ):
        self.content = content
        self._file_key = file_key
        self.mime = mime
        self.parent = parent
        self.file_info = (
            file_info
            if isinstance(file_info, FileInfoModel)
            else FileInfoModel(**file_info)
        )

    @property
    def file_key(self):
        if self.file_info:
            sku = self.parent.sku if self.parent else ""
            prefix = self.file_info.prefix or ""
            return f"{prefix}.{sku}.{self.file_info.id}.{self.file_info.format.lower()}"
        else:
            return self._file_key



from gdshoplib.services.vk.vk import VK


class VKMarket:
    def __init__(self, manager=None):
        self.manager = manager or VK()

    def list(self):
        return self.manager.request(
            "market.get",
            params={
                "owner_id": f"-{self.manager.settings.VK_GROUP_ID}",
                "extended": 1,
                "with_disabled": 1,
                "need_variants": 1,
            },
        )

    def get(self, sku):
        ...

    def add(self, sku):
        ...

    def edit(self, sku):
        ...

    def delete(self, sku):
        ...

    def album_get(self, sku):
        ...

    def album_create(self, sku):
        ...

    def album_edit(self, sku):
        ...

    def album_delete(self, sku):
        ...

    def album_add(self, sku):
        ...



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gdshoplib',
 'gdshoplib.apps',
 'gdshoplib.apps.finance',
 'gdshoplib.apps.marketing',
 'gdshoplib.apps.platforms',
 'gdshoplib.apps.products',
 'gdshoplib.cli',
 'gdshoplib.core',
 'gdshoplib.packages',
 'gdshoplib.services',
 'gdshoplib.services.notion',
 'gdshoplib.services.vk']

package_data = \
{'': ['*'], 'gdshoplib': ['templates/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'Pillow>=9.3.0,<10.0.0',
 'boto3>=1.26.32,<2.0.0',
 'isort>=5.10.1,<6.0.0',
 'lxml>=4.9.1,<5.0.0',
 'orjson>=3.8.3,<4.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'python-magic>=0.4.27,<0.5.0',
 'redis>=4.3.5,<5.0.0',
 'requests>=2.28.1,<3.0.0',
 'rich>=12.6.0,<13.0.0',
 'twine>=4.0.1,<5.0.0',
 'typer[all]>=0.7.0,<0.8.0',
 'validators>=0.20.0,<0.21.0']

entry_points = \
{'console_scripts': ['gdshoplib = gdshoplib:application']}

setup_kwargs = {
    'name': 'gdshoplib',
    'version': '1.11.3',
    'description': '',
    'long_description': '# Библиотека для управления магазином gdshop\n\nПользовательские объекты для высокоуровневой работы без привязки к слою данных. Предоставляет Python like интерфейс для работы с объектами\n\n## Finance\n\n- [ ] Информация о состоянии счетов\n- [ ] Операций с финансами\n- [ ] Статистика по финансам за периоды + Прогнозы\n- [ ] Расчет и контроль бюджетов\n- [ ] Сбор информации с площадок по финансам\n- [ ] Генерация отчетов\n- [x] Генерация цен по коэфицентам\n\n## Marketing\n\n- [ ] Статистика рекламных компаний\n- [ ] Управление рекламными компаниями\n- [ ] Статистика воронок\n- [ ] Выгрузка информации с площадок\n- [ ] Генерация отчетов\n- [ ] Статистика по товарам\n\n## Order\n- [ ] Получение информации о заказах\n\n## Product\n- [x] Получение информации о продукте\n- [ ] Обновление содержимого продукта\n- [x] Выгрузка информации о товаре в фид\n- [ ] Работа с закупками\n- [ ] Оценка материалов\n- [x] Работа с media\n- [x] Работа с description\n- [x] Выгрузка меда в S3\n\n## Platform\n- [x] Генерация товарных фидов\n\n\n# Полезные ссылки\n\n## Описание товарного фида\n\nhttps://yandex.ru/support/partnermarket/export/yml.html\nhttps://drive.google.com/file/d/1kkKa0KU7iNOszyC2oQSGmHNwp3sRGKFb/view',
    'author': 'Nikolay Baryshnikov',
    'author_email': 'root@k0d.ru',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/p141592',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
